#include "stdafx.h"

DWORD (*_ptr_GetASPI32SupportInfo)(VOID);
DWORD (*_ptr_SendASPI32Command)(LPSRB);

// Initializes WNASPI32.DLL
// It really does not matter that this doesn't intend on ever freeing the
// library.  It even discards the return value of LoadLibrary.
DWORD InitASPI(void)
{
	HINSTANCE handle;
	DWORD temp;

	handle = LoadLibrary (WNASPI32_DLL);
	if (!handle)
	{
		temp = GetLastError();
		return (!temp) ? 1 : temp;
	}

	_ptr_GetASPI32SupportInfo = (DWORD (*)(VOID)) GetProcAddress(handle,
		"GetASPI32SupportInfo");
	_ptr_SendASPI32Command = (DWORD (*)(LPSRB)) GetProcAddress(handle,
		"SendASPI32Command");

	if (!_ptr_GetASPI32SupportInfo || !_ptr_SendASPI32Command)
		return 1;

	return 0;
}

// Retrieve the drive type and such for a single drive and store it in
// device->info
static void GetDriveInfo(Device *device)
{
	static const BYTE modePage = 0x2A;
	static const DWORD resultError = 0xDEADBEEF;
	static const BYTE READ_DVD_ROM = 0x10;
	static const BYTE SCSI_REPORT_KEY = 0xA4;
	static const BYTE GET_RPC = 0x08;

	BYTE CDB[12];
	BYTE data[64];
	DWORD result;

	// Get the abilities of the drive
	CDB[0] = SCSI_MODE_SEN10;
	CDB[1] = 0x00; // no DBD
	CDB[2] = 0x00 | modePage;
	CDB[3] = 0x00; // these are reserved
	CDB[4] = 0x00;
	CDB[5] = 0x00;
	CDB[6] = 0x00;
	CDB[7] = (BYTE) (sizeof(data) >> 8); // allocation length MSB
	CDB[8] = (BYTE) (sizeof(data) & 0xFF); // allocation length LSB
	CDB[9] = 0x00; // control?

	// Execute command
	result = device->SendCommand(CDB, 10, SRB_DIR_IN, data, sizeof(data));

	// The first 8 bytes are a header saying how big the next part is.  Only
	// the first 2 bytes of this header are of any interest to us, but we
	// have to add 8 to all offsets later.
	// Sanity check
	if (!result && (((data[0] << 8) | data[1]) < 12))
		result = resultError;

	// Flag errors
	if (result)
	{
		device->info.error = true;
		device->info.errorCode = result;
		return;
	}

	// No errors.
	device->info.error = false;

	// Is it not a DVD-ROM drive?
	if (!(data[8 + 2] & READ_DVD_ROM))
	{
		device->info.isDVDROM = false;
		return;
	}

	// We know it's a DVD-ROM.
	device->info.isDVDROM = true;

	// Now check RPC status.
	CDB[0] = SCSI_REPORT_KEY;
	CDB[1] = 0x00; // reserved
	CDB[2] = 0x00; // LBA (none so reserved)
	CDB[3] = 0x00;
	CDB[4] = 0x00;
	CDB[5] = 0x00;
	CDB[6] = 0x00; // reserved
	CDB[7] = 0x00; // reserved
	CDB[8] = (BYTE) (sizeof(data) >> 8); // allocation length MSB
	CDB[9] = (BYTE) (sizeof(data) & 0xFF); // allocation length LSB
	CDB[10] = GET_RPC; // AGID (unused) and key type (get RPC)
	CDB[11] = 0x00; // control

	result = device->SendCommand(CDB, 12, SRB_DIR_IN, data, sizeof(data));

	// If there's an error, it's RPC1.  It may also be a Hitachi drive.
	if (result)
	{
		device->info.isRPC2 = false;
		return;
	}

	// Sanity check
	if (((data[0] << 8) | data[1]) < 6)
	{
		device->info.isRPC2 = false;
		return;
	}

	// Check for a valid return saying it's RPC1
	if (data[6] == 0x00)
	{
		device->info.isRPC2 = false;
		return;
	}

	// Drive is RPC2
	device->info.isRPC2 = true;
}


// Refresh the drive information for all drives
void RefreshDriveInfo (void)
{
	Device *device;

	for (device = devices; device; device = device->next)
	{
		GetDriveInfo(device);
	}
}

