#include "stdafx.h"

Device *devices = NULL;
int deviceCount = 0;
HANDLE event = NULL;

// Make a list of available SCSI devices
DWORD Device::makeDeviceList(void)
{
	// Delete the current devices if any are there
	if (devices)
	{
		Device *temp = devices, *p;

		for (; temp; temp = p)
		{
			p = temp->next;
			delete temp;
		}

		devices = NULL;
		deviceCount = 0;
	}

	DWORD status = GetASPI32SupportInfo();
	int tempCount = LOBYTE(status);

	deviceCount = 0;

	// SCSI error occurred?
	if (HIBYTE(LOWORD(status)) != SS_COMP)
		return (DWORD) -1;

	// Are there no adaptors?  This isn't an error condition.
	if (!tempCount)
		return 0;

	// Loop to create list of devices
	int adaptorID, targetID, unitID;
	BYTE CDB[16];
	Inquiry inq;
	Device *dev;
	DWORD error;

	for (adaptorID = (tempCount - 1); adaptorID >= 0; adaptorID--)
		for (targetID = (MAX_ID - 1); targetID >= 0; targetID--)
			for (unitID = (MAX_ID - 1); unitID >= 0; unitID--)
			{
				// Create a new Device and initialize it
				dev = new Device();

				dev->HaId = adaptorID;
				dev->Target = targetID;
				dev->Lun = unitID;
				memset((void *) &inq, 0, sizeof(Inquiry));

				// Send SCSI command
				memset((void *) CDB, 0, sizeof(CDB));
				CDB[0] = SCSI_INQUIRY;
				CDB[4] = sizeof(Inquiry);

				if (!!(error = dev->SendCommand(CDB, 6, SRB_DIR_IN,
					(BYTE *) &inq, sizeof(Inquiry))))
				{
					delete dev;

					// No device is a normal error
					continue;
				}

				// Is it a CD-ROM?
				if ((inq.devType != DTYPE_CROM) && (inq.devType !=
					DTYPE_WORM))
				{
					delete dev;
					continue;
				}

				// Add device to list
				memcpy(dev->name, inq.Vendor, 49);
				dev->name[49] = 0;

				deviceCount++;

				dev->next = devices;
				dev->prev = NULL;
				if (devices)
					devices->prev = dev;
				devices = dev;
			}
	return 0;
}

// Sends an ASPI command
DWORD Device::SendCommand(BYTE *CDBByte, BYTE CDBLen, BYTE SRB_Flags,
	BYTE *BufPointer, DWORD BufLen) const
{
	// Create the event if it doesn't exist yet
	if (!event)
		event = CreateEvent(NULL, TRUE, FALSE, NULL);
	ResetEvent(event);

	// Initialize structure
	SRB_ExecSCSICmd srb;

	memset((void *) &srb, 0, sizeof(srb));

	srb.SRB_Cmd = SC_EXEC_SCSI_CMD;
	srb.SRB_HaId = this->HaId;
	srb.SRB_Flags = SRB_Flags | SRB_EVENT_NOTIFY;
	srb.SRB_Target = this->Target;
	srb.SRB_Lun = this->Lun;
	srb.SRB_BufLen = BufLen;
	srb.SRB_BufPointer = BufPointer;
	srb.SRB_SenseLen = SENSE_LEN;
	srb.SRB_CDBLen = CDBLen;
	srb.SRB_PostProc = (void *) event;
	
	memcpy((void *) srb.CDBByte, (void *) CDBByte, CDBLen);


	// Execute command
	DWORD ASPIStatus = SendASPI32Command(&srb);

	if (ASPIStatus == SS_PENDING)
		WaitForSingleObject(event, INFINITE);


	// Check for errors
	if (srb.SRB_Status != SS_COMP)
	{
		DWORD temp = ((srb.SenseArea[2] & 0x0F) << 16) | (srb.SenseArea[12]
			<< 8) | (srb.SenseArea[13]);

		// If sense data makes no sense
		if (!temp)
			temp = (srb.SRB_Status << 24) | (srb.SRB_HaStat << 16) |
				(srb.SRB_TargStat << 8);

		if (!temp)
			temp = 0x04000000;
		
		return temp;
	}

	return 0;
}


